@@ -100,13 +100,15 @@ impl Drop for Lua {
     fn drop(&mut self) {
         unsafe {
             if !self.ephemeral {
-                let mut extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
+                let extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
                 mlua_debug_assert!(
                     ffi::lua_gettop(extra.ref_thread) == extra.ref_stack_max
                         && extra.ref_stack_max as usize == extra.ref_free.len(),
                     "reference leak detected"
                 );
-                *mlua_expect!(extra.registry_unref_list.lock(), "unref list poisoned") = None;
+                let mut unref_list =
+                    mlua_expect!(extra.registry_unref_list.lock(), "unref list poisoned");
+                *unref_list = None;
                 ffi::lua_close(self.main_state.expect("main_state is null"));
                 if !extra.mem_info.is_null() {
                     Box::from_raw(extra.mem_info);
@@ -490,7 +492,7 @@ impl Lua {
         unsafe { (*extra.mem_info).used_memory as usize }
     }
 
-    /// Sets a memory limit on this Lua state.
+    /// Sets a memory limit (in bytes) on this Lua state.
     ///
     /// Once an allocation occurs that would pass this memory limit,
     /// a `Error::MemoryError` is generated instead.
@@ -658,6 +660,9 @@ impl Lua {
     /// similar on the returned builder.  Code is not even parsed until one of these methods is
     /// called.
     ///
+    /// If this `Lua` was created with `unsafe_new`, `load` will automatically detect and load
+    /// chunks of either text or binary type, as if passing `bt` mode to `luaL_loadbufferx`.
+    ///
     /// [`Chunk::exec`]: struct.Chunk.html#method.exec
     pub fn load<'lua, 'a, S>(&'lua self, source: &'a S) -> Chunk<'lua, 'a>
     where
@@ -668,6 +673,7 @@ impl Lua {
             source: source.as_ref(),
             name: None,
             env: None,
+            mode: None,
         }
     }
 
@@ -676,28 +682,35 @@ impl Lua {
         source: &[u8],
         name: Option<&CString>,
         env: Option<Value<'lua>>,
+        mode: Option<ChunkMode>,
     ) -> Result<Function<'lua>> {
         unsafe {
             let _sg = StackGuard::new(self.state);
             assert_stack(self.state, 1);
 
-            match if let Some(name) = name {
-                ffi::luaL_loadbufferx(
-                    self.state,
-                    source.as_ptr() as *const c_char,
-                    source.len(),
-                    name.as_ptr() as *const c_char,
-                    cstr!("t"),
-                )
-            } else {
-                ffi::luaL_loadbufferx(
-                    self.state,
-                    source.as_ptr() as *const c_char,
-                    source.len(),
-                    ptr::null(),
-                    cstr!("t"),
-                )
-            } {
+            let mode_str = match mode {
+                Some(ChunkMode::Binary) if self.safe => {
+                    return Err(Error::SafetyError(
+                        "binary chunks are disabled in safe mode".to_string(),
+                    ))
+                }
+                Some(ChunkMode::Binary) => cstr!("b"),
+                Some(ChunkMode::Text) => cstr!("t"),
+                None if source.starts_with(ffi::LUA_SIGNATURE) && self.safe => {
+                    return Err(Error::SafetyError(
+                        "binary chunks are disabled in safe mode".to_string(),
+                    ))
+                }
+                None => cstr!("bt"),
+            };
+
+            match ffi::luaL_loadbufferx(
+                self.state,
+                source.as_ptr() as *const c_char,
+                source.len(),
+                name.map(|n| n.as_ptr()).unwrap_or_else(ptr::null),
+                mode_str,
+            ) {
                 ffi::LUA_OK => {
                     if let Some(env) = env {
                         self.push_value(env)?;
@@ -1274,11 +1287,10 @@ impl Lua {
     /// by `Lua::remove_registry_value`.
     pub fn expire_registry_values(&self) {
         unsafe {
-            let mut extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
-            let unref_list = mem::replace(
-                &mut *mlua_expect!(extra.registry_unref_list.lock(), "unref list poisoned"),
-                Some(Vec::new()),
-            );
+            let extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
+            let mut unref_list =
+                mlua_expect!(extra.registry_unref_list.lock(), "unref list poisoned");
+            let unref_list = mem::replace(&mut *unref_list, Some(Vec::new()));
             for id in mlua_expect!(unref_list, "unref list not set") {
                 ffi::luaL_unref(self.state, ffi::LUA_REGISTRYINDEX, id);
             }
@@ -1802,6 +1814,14 @@ pub struct Chunk<'lua, 'a> {
     source: &'a [u8],
     name: Option<CString>,
     env: Option<Value<'lua>>,
+    mode: Option<ChunkMode>,
+}
+
+/// Represents chunk mode (text or binary).
+#[derive(Clone, Copy)]
+pub enum ChunkMode {
+    Text,
+    Binary,
 }
 
 impl<'lua, 'a> Chunk<'lua, 'a> {
@@ -1833,6 +1853,17 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
         Ok(self)
     }
 
+    /// Sets whether the chunk is text or binary (autodetected by default).
+    ///
+    /// Lua does not check the consistency of binary chunks, therefore this mode is allowed only
+    /// for instances created with [`Lua::unsafe_new`].
+    ///
+    /// [`Lua::unsafe_new`]: struct.Lua.html#method.unsafe_new
+    pub fn set_mode(mut self, mode: ChunkMode) -> Chunk<'lua, 'a> {
+        self.mode = Some(mode);
+        self
+    }
+
     /// Execute this chunk of code.
     ///
     /// This is equivalent to calling the chunk function with no arguments and no return values.
@@ -1862,13 +1893,17 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
     /// the value that it evaluates to.  Otherwise, the chunk is interpreted as a block as normal,
     /// and this is equivalent to calling `exec`.
     pub fn eval<R: FromLuaMulti<'lua>>(self) -> Result<R> {
-        // First, try interpreting the lua as an expression by adding
+        // Bytecode is always interpreted as a statement.
+        // For source code, first try interpreting the lua as an expression by adding
         // "return", then as a statement.  This is the same thing the
         // actual lua repl does.
-        if let Ok(function) = self.lua.load_chunk(
+        if self.source.starts_with(ffi::LUA_SIGNATURE) {
+            self.call(())
+        } else if let Ok(function) = self.lua.load_chunk(
             &self.expression_source(),
             self.name.as_ref(),
             self.env.clone(),
+            self.mode,
         ) {
             function.call(())
         } else {
@@ -1889,10 +1924,13 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
         'lua: 'fut,
         R: FromLuaMulti<'lua> + 'fut,
     {
-        if let Ok(function) = self.lua.load_chunk(
+        if self.source.starts_with(ffi::LUA_SIGNATURE) {
+            self.call_async(())
+        } else if let Ok(function) = self.lua.load_chunk(
             &self.expression_source(),
             self.name.as_ref(),
             self.env.clone(),
+            self.mode,
         ) {
             function.call_async(())
         } else {
@@ -1932,7 +1970,7 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
     /// This simply compiles the chunk without actually executing it.
     pub fn into_function(self) -> Result<Function<'lua>> {
         self.lua
-            .load_chunk(self.source, self.name.as_ref(), self.env)
+            .load_chunk(self.source, self.name.as_ref(), self.env, self.mode)
     }
 
     fn expression_source(&self) -> Vec<u8> {
