@@ -5,7 +5,8 @@
 
 #[cfg_attr(
     all(feature = "luajit", target_os = "macos", target_arch = "x86_64"),
-    link_args = "-pagezero_size 10000 -image_base 100000000"
+    link_args = "-pagezero_size 10000 -image_base 100000000",
+    allow(unused_attributes)
 )]
 extern "system" {}
 
